{
  (C) 2003 Dirk Frischalowski, dfrischalowski@del-net.com, all rights reserved.

  Version 1.0 25.02.2003
}

unit DFCRC32;

interface

uses
  Classes, SysUtils;

type
  DFCheckSFVFile = procedure(FileName: string; OK: boolean);

function  DFCalcCRC32(FileName: string): string;
procedure DFCreateSFVFile(FileList: TStringList; FileName, Header: string; WinSFVCompatible: Boolean);
function  DFCheckSFV(Directory, SFVFileName: string; CallBackPerFile: DFCheckSFVFile): boolean;

implementation

var
  DFCRCTable: array[0..512] of longint;

type
  DFLong = record
    LoWord: word;
    HiWord: word;
  end;

const
  DFCRCPOLY = $EDB88320;

procedure BuildCRCTable;
var
  i, j: word;
  code: longint;
begin
  FillChar(DFCRCTable, SizeOf(DFCRCTable), 0);
  for i := 0 to 255 do
  begin
    code := i shl 1;
    for j := 8 downto 0 do
      if (code and 1) <> 0 then
        code := (code shr 1) xor DFCRCPOLY
      else
        code := code shr 1;
    DFCRCTable[i] := code;
   end;
end;

function RecountCRC(b: byte; OldCrc: longint): Longint;
begin
  Result := DFCRCTable[byte(OldCrc xor longint(b))] xor ((OldCrc shr 8) and $00FFFFFF)
end;

function HexW(w: word): string;
const
  HexChar: array[0..15] of char = '0123456789ABCDEF';
begin
  Result := '';
  Result := HexChar[Hi(w) shr 4] + HexChar[Hi(w) and $F] +
            HexChar[Lo(w) shr 4] + HexChar[Lo(w) and $F];
end;

function HexL(l: longint): string;
begin
  with DFLong(l) do
    HexL := HexW(HiWord) + HexW(LoWord);
end;

function DFCalcCRC32(FileName: string): string;
var
  Buffer: PChar;
  F: File of byte;
  B: array[0..255] of byte;
  CRC: longint;
  BytesRead, i: integer;
begin
  Result := '';
  if FileExists(FileName) then
  begin
    CRC := $FFFFFFFF;
    AssignFile(F, FileName);
    FileMode := 0;
    {$I+}
    try
      Reset(F);
      GetMem(Buffer, sizeof(B));
      try
        repeat
          FillChar(B, sizeof(B), 0);
          BlockRead(F, B, sizeof(B), BytesRead);
          for i := 0 to (BytesRead - 1) do
           CRC := RecountCRC(B[i], CRC);
        until (BytesRead < 255);
      finally
        FreeMem(Buffer, sizeof(B));
      end;
    finally
      CloseFile(F);
    end;
    {$I-}
    CRC := Not CRC;
    Result := HexL(CRC);
  end;
end;

procedure DFCreateSFVFile(FileList: TStringList; FileName, Header: string; WinSFVCompatible: Boolean);
var
  vSL: TStringList;
  Cnt: integer;
begin
  vSL := TStringList.Create;
  try
    if WinSFVCompatible then
      vSL.Add('; Generated by SFV32 v1 (WinSFV Compat)');
    vSL.Text := Header;
    for Cnt := 0 to FileList.Count - 1 do
      vSL.Add(ExtractFileName(FileList[Cnt]) + ' ' + DFCalcCRC32(FileList[Cnt]));
    vSL.SaveToFile(FileName);
  finally
    vSL.Free;
  end;
end;

function DFCheckSFV(Directory, SFVFileName: string; CallBackPerFile: DFCheckSFVFile): boolean;
var
  vSL: TStringList;
  Cnt: integer;
  FileName: string;
begin
  Result := false;
  if Directory[Length(Directory)] <> '\' then
    Directory := Directory + '\';

  vSL := TStringList.Create;
  try
    vSL.LoadFromFile(SFVFileName);
    for Cnt := 0 to vSL.Count - 1 do
      vSL[Cnt] := Trim(vSL[Cnt]);
    for Cnt := 0 to vSL.Count - 1 do
    begin
      if vSL[Cnt][1] <> ';' then
      begin
        FileName := Trim(Copy(vSL[Cnt], 1, Length(vSL[Cnt]) - 8));
        if Copy(vSL[Cnt], Length(vSL[Cnt]) - 7, 8) <> DFCalcCRC32(Directory + FileName) then
        begin
          if assigned(CallBackPerFile) then
            CallBackPerFile(Directory + FileName, false)
          else
            exit;
        end
        else
          if assigned(CallBackPerFile) then
            CallBackPerFile(Directory + FileName, true);
      end;
    end;
  finally
    vSL.Free;
  end;
  Result := true;
end;

initialization
  BuildCRCTable;
end.
